# pylint: skip-file

import glob
import logging
from timeit import default_timer as timer

import webrtcvad  # type: ignore
from deepspeech import Model  # type: ignore

import pydeepspeech.wavSplit as wavSplit

"""
Load the pre-trained model into the memory
@param models: Output Grapgh Protocol Buffer file
@param scorer: Scorer file

@Retval
Returns a list [DeepSpeech Object, Model Load Time, Scorer Load Time]
"""


def load_model(models, scorer):
    model_load_start = timer()
    ds = Model(models)
    model_load_end = timer() - model_load_start
    logging.debug("Loaded model in %0.3fs." % (model_load_end))

    scorer_load_start = timer()
    ds.enableExternalScorer(scorer)
    scorer_load_end = timer() - scorer_load_start
    logging.debug("Loaded external scorer in %0.3fs." % (scorer_load_end))

    return [ds, model_load_end, scorer_load_end]


"""
Run Inference on input audio file
@param ds: Deepspeech object
@param audio: Input audio for running inference on
@param fs: Sample rate of the input audio file

@Retval:
Returns a list [Inference, Inference Time, Audio Length]

"""


def stt(ds, audio, fs):
    inference_time = 0.0
    audio_length = len(audio) * (1 / fs)

    # Run Deepspeech
    logging.debug("Running inference...")
    inference_start = timer()
    output = ds.stt(audio)
    inference_end = timer() - inference_start
    inference_time += inference_end
    logging.debug(
        "Inference took %0.3fs for %0.3fs audio file."
        % (inference_end, audio_length)
    )

    return [output, inference_time]


"""
Resolve directory path for the models and fetch each of them.
@param dirName: Path to the directory containing pre-trained models

@Retval:
Retunns a tuple containing each of the model files (pb, scorer)
"""


def resolve_models(dirName):
    pb = glob.glob(dirName + "/*.pbmm")[0]
    logging.debug("Found Model: %s" % pb)

    scorer = glob.glob(dirName + "/*.scorer")[0]
    logging.debug("Found scorer: %s" % scorer)

    return pb, scorer


"""
Generate VAD segments. Filters out non-voiced audio frames.
@param waveFile: Input wav file to run VAD on.0

@Retval:
Returns tuple of
    segments: a bytearray of multiple smaller audio frames
              (The longer audio split into mutiple smaller one's)
    sample_rate: Sample rate of the input audio file
    audio_length: Duraton of the input audio file

"""


def vad_segment_generator(wavFile, aggressiveness):
    logging.debug("Caught the wav file @: %s" % (wavFile))
    audio, sample_rate, audio_length = wavSplit.read_wave(wavFile)
    assert (
        sample_rate == 16000
    ), "Only 16000Hz input WAV files are supported for now!"
    vad = webrtcvad.Vad(int(aggressiveness))
    frames = wavSplit.frame_generator(30, audio, sample_rate)
    frames = list(frames)
    segments = wavSplit.vad_collector(sample_rate, 30, 300, vad, frames)

    return segments, sample_rate, audio_length



import argparse

from pydeepspeech.install_models import install_deepspeechmodules


def main():
    args = argparse.ArgumentParser(
        description="Installs pbmm and scorer files for pydeepspeech",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    args.add_argument("--pbmm", required=True, help="Path to the pbmm file")
    args.add_argument("--scorer", required=True, help="Path to the scorer file")
    args = args.parse_args()
    url_pbmm = args.pbmm or input("Path or URL to the pbmm file: ")
    url_scorer = args.pbmm or input("Path or URL to the scorer file: ")
    install_deepspeechmodules(url_pbmm=url_pbmm, url_scorer=url_scorer)


if __name__ == "__main__":
    main()



import argparse
import os
import shutil
import sys

from pydeepspeech.install_models import install_dependencies_if_necessary
from pydeepspeech.transcribe import transcribe


def check_path(path: str):
    if os.path.exists(path):
        return
    raise OSError(f"Did not find expected path {path}")


def main() -> None:
    parser = argparse.ArgumentParser(
        description="Transcribe long audio files using webRTC VAD or use the streaming interface"
    )
    parser.add_argument(
        "--aggressive",
        type=int,
        choices=range(4),
        required=False,
        default=1,
        help="Determines how aggressive filtering out non-speech is. (Interger between 0-3)",
    )
    parser.add_argument(
        "--wav_file",
        required=True,
        help="Path to the audio file to run (WAV format)",
    )
    parser.add_argument("--out_file", default=None)
    parser.add_argument(
        "--model_dir",
        help="Path to directory that contains all model files (output_graph and scorer)",
        default=None,
    )
    args = parser.parse_args()
    model_dir = args.model_dir or install_dependencies_if_necessary()
    expected_txt_file = args.wav_file[:-4] + ".txt"
    check_path(args.wav_file)
    check_path(model_dir)
    check_path(os.path.join(model_dir, "deepspeech-0.9.3-models.scorer"))
    check_path(os.path.join(model_dir, "deepspeech-0.9.3-models.pbmm"))
    transcribe(aggressive=args.aggressive, audio=args.wav_file, model=model_dir)
    if not os.path.exists(expected_txt_file):
        print(
            f"Expected a generated text file at {expected_txt_file} but none appeared."
        )
        sys.exit(1)
    if args.out_file:
        shutil.move(expected_txt_file, args.out_file)
        expected_txt_file = args.out_file
    print(f"Wrote out subtitle file {expected_txt_file}")
    sys.exit(0)


if __name__ == "__main__":
    main()



# type: ignore

import os
import shutil
import threading
from pathlib import Path
from typing import Optional

import requests

from pydeepspeech.util import get_appdatadir

# AI model used for the application
_VERSION = "v0.9.3"

URL_PBMM = "https://github.com/mozilla/DeepSpeech/releases/download/v0.9.3/deepspeech-0.9.3-models.pbmm"

URL_SCORER = "https://github.com/mozilla/DeepSpeech/releases/download/v0.9.3/deepspeech-0.9.3-models.scorer"

MODEL_DIR = os.path.join(get_appdatadir(), "model", _VERSION)
# Marks the model created.
IS_FINISHED_STAMP = os.path.join(MODEL_DIR, "is_finished")


def download_file(url, outfile) -> None:
    if os.path.isfile(url):
        # Actually it's a file, so we can just copy it.
        shutil.copyfile(url, outfile)
        return
    # NOTE the stream=True parameter below
    try:
        tmp = f"{outfile}.tmp"
        if os.path.exists(tmp):
            os.remove(tmp)
        with requests.get(url, stream=True) as r:
            r.raise_for_status()
            with open(tmp, "wb") as f:
                for chunk in r.iter_content(chunk_size=8192):
                    # If you have chunk encoded response uncomment if
                    # and set chunk_size parameter to None.
                    # if chunk:
                    f.write(chunk)
        os.rename(tmp, outfile)
    except KeyboardInterrupt:
        print(f"Aborted download of {url}")
        return


def url_to_local_name(url: str) -> str:
    return os.path.join(MODEL_DIR, url.split("/")[-1])


def is_models_installed() -> bool:
    return os.path.exists(IS_FINISHED_STAMP)


def install_deepspeechmodules(
    url_pbmm: Optional[str] = URL_PBMM,
    url_scorer: Optional[str] = URL_SCORER,
) -> None:
    if os.path.exists(IS_FINISHED_STAMP):
        os.remove(IS_FINISHED_STAMP)
    os.makedirs(MODEL_DIR, exist_ok=True)
    threads = {}
    if os.path.exists(IS_FINISHED_STAMP):
        return
    url_pbmm = url_pbmm or URL_PBMM
    url_scorer = url_scorer or URL_SCORER
    print(
        "Downloading and installing the models for the first time. This may take a while."
    )
    for url in [url_pbmm, url_scorer]:
        local_filename = url_to_local_name(url)
        t = threading.Thread(target=download_file, args=(url, local_filename))
        print(f"Downloading {url} -> {local_filename}")
        threads[url] = t
        t.daemon = True
        t.start()
    for url, t in threads.items():
        while t.is_alive():  # allows keyboard interrupt.
            t.join(0.2)
        print(f"Finished downloading {url}")
    Path(IS_FINISHED_STAMP).touch()


def install_dependencies_if_necessary() -> str:  # pylint: disable=invalid-name
    print(f"Model directory is: {MODEL_DIR}")
    if not is_models_installed():
        install_deepspeechmodules(url_pbmm=URL_PBMM, url_scorer=URL_SCORER)
    return MODEL_DIR


if __name__ == "__main__":
    install_dependencies_if_necessary()



# type: ignore
import os
from distutils.version import LooseVersion

import pip

if LooseVersion(pip.__version__) < LooseVersion("10"):
    # older pip version
    from pip.utils.appdirs import (  # pylint: disable=import-error,no-name-in-module
        user_cache_dir,
    )
else:
    # newer pip version
    from pip._internal.utils.appdirs import user_cache_dir


# create your program's directory
def get_appdatadir() -> str:
    return os.path.join(user_cache_dir("pip"), "pydeepspeech_models")



import logging
import os
import shlex
import subprocess

import numpy as np  # pylint: disable=import-error

import pydeepspeech.wavTranscriber as wavTranscriber


def transcribe(aggressive, audio, model):
    # Point to a path containing the pre-trained models & resolve ~ if used
    dir_name = os.path.expanduser(model)

    # Resolve all the paths of model files
    output_graph, scorer = wavTranscriber.resolve_models(dir_name)

    # Load output_graph, alpahbet and scorer
    model_retval = wavTranscriber.load_model(output_graph, scorer)

    if audio is not None:
        title_names = [
            "Filename",
            "Duration(s)",
            "Inference Time(s)",
            "Model Load Time(s)",
            "Scorer Load Time(s)",
        ]
        print(
            "\n%-30s %-20s %-20s %-20s %s"
            % (
                title_names[0],
                title_names[1],
                title_names[2],
                title_names[3],
                title_names[4],
            )
        )

        inference_time = 0.0

        # Run VAD on the input file
        wave_file = audio
        (
            segments,
            sample_rate,
            audio_length,
        ) = wavTranscriber.vad_segment_generator(wave_file, aggressive)
        f = open(wave_file.rstrip(".wav") + ".txt", "w")
        logging.debug("Saving Transcript @: %s" % wave_file.rstrip(".wav") + ".txt")

        for i, segment in enumerate(segments):
            # Run deepspeech on the chunk that just completed VAD
            logging.debug("Processing chunk %002d" % (i,))
            audio = np.frombuffer(segment, dtype=np.int16)
            output = wavTranscriber.stt(model_retval[0], audio, sample_rate)
            inference_time += output[1]
            logging.debug("Transcript: %s" % output[0])

            f.write(output[0] + " ")

        # Summary of the files processed
        f.close()

        # Extract filename from the full file path
        filename, ext = os.path.split(os.path.basename(wave_file))
        logging.debug(
            "************************************************************************************************************"
        )
        logging.debug(
            "%-30s %-20s %-20s %-20s %s"
            % (
                title_names[0],
                title_names[1],
                title_names[2],
                title_names[3],
                title_names[4],
            )
        )
        logging.debug(
            "%-30s %-20.3f %-20.3f %-20.3f %-0.3f"
            % (
                filename + ext,
                audio_length,
                inference_time,
                model_retval[1],
                model_retval[2],
            )
        )
        logging.debug(
            "************************************************************************************************************"
        )
        print(
            "%-30s %-20.3f %-20.3f %-20.3f %-0.3f"
            % (
                filename + ext,
                audio_length,
                inference_time,
                model_retval[1],
                model_retval[2],
            )
        )
    else:
        sctx = model_retval[0].createStream()
        subproc = subprocess.Popen(
            shlex.split("rec -q -V0 -e signed -L -c 1 -b 16 -r 16k -t raw - gain -2"),
            stdout=subprocess.PIPE,
            bufsize=0,
        )
        print("You can start speaking now. Press Control-C to stop recording.")

        try:
            while True:
                data = subproc.stdout.read(512)
                sctx.feedAudioContent(np.frombuffer(data, np.int16))
        except KeyboardInterrupt:
            print("Transcription: ", sctx.finishStream())
            subproc.terminate()
            subproc.wait()



import os
import sys
from shutil import rmtree

from setuptools import Command, find_packages, setup

# The directory containing this file
HERE = os.path.dirname(__file__)

NAME = "pydeepspeech"
DESCRIPTION = "Mozilla's DeepSpeech transcriber in a pip installable package."
URL = "https://github.com/zackees/pydeepspeech"
EMAIL = "dont@email.me"
AUTHOR = "Zach Vorhies"
REQUIRES_PYTHON = ">=3.6.0"
VERSION = "1.1.4"

# The text of the README file
with open(os.path.join(HERE, "README.md")) as fd:
    README = fd.read()


class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        pass

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(HERE, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")
        os.system(
            "{0} setup.py sdist bdist_wheel --universal".format(sys.executable)
        )

        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")

        self.status("Pushing git tags…")
        os.system("git tag v{0}".format(VERSION))
        os.system("git push --tags")

        sys.exit()


# This call to setup() does all the work
setup(
    name=NAME,
    python_requires=REQUIRES_PYTHON,
    version=VERSION,
    description=DESCRIPTION,
    long_description=README,
    long_description_content_type="text/markdown",
    url=URL,
    author=AUTHOR,
    author_email=EMAIL,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.8",
    ],
    install_requires=[
        "deepspeech==0.9.3",
        "webrtcvad",
        "pathlib",
        "capturing_process",
        "requests",
        "black",
    ],
    entry_points={
        "console_scripts": [
            "pydeepspeech = pydeepspeech.run:main",
            "pydeepspeech_installmodels = pydeepspeech.interactive_install_modules:main",
        ],
    },
    packages=find_packages(
        exclude=["tests", "*.tests", "*.tests.*", "tests.*"]
    ),
    include_package_data=True,
    extras_require={
        "test": ["pytest"],
    },
    cmdclass={
        "upload": UploadCommand,
    },
)



import argparse
import logging
import sys

from pydeepspeech.transcribe import transcribe

# Debug helpers
logging.basicConfig(stream=sys.stderr, level=logging.CRITICAL)


def main(args):
    parser = argparse.ArgumentParser(
        description="Transcribe long audio files using webRTC VAD or use the streaming interface"
    )
    parser.add_argument(
        "--aggressive",
        type=int,
        choices=range(4),
        required=False,
        help="Determines how aggressive filtering out non-speech is. (Interger between 0-3)",
    )
    parser.add_argument(
        "--audio",
        required=False,
        help="Path to the audio file to run (WAV format)",
    )
    parser.add_argument(
        "--model",
        required=True,
        help="Path to directory that contains all model files (output_graph and scorer)",
    )
    args = parser.parse_args(args)
    if args.audio is not None:
        logging.debug("Transcribing audio file @ %s" % args.audio)
    else:
        parser.print_help()
        parser.exit()
    transcribe(args.aggressive, args.audio, args.model)


if __name__ == "__main__":
    main(sys.argv[1:])
